#!/usr/bin/env python
import os
import sys
import multiprocessing as mp
from subprocess import PIPE, Popen
import subprocess
from collections import Iterable
import numpy as np
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Routines.Alignment import AlignmentRoutines


print_mutex = mp.Lock()
SHELLPATH = "/bin/bash"

def execute(exe_string):
    # this function is global because of stutid damned pickle mode in python!!!!!
    # use mutex to safe write to stdout from multiple threads
    print_mutex.acquire()
    sys.stdout.write("Executing:\n\t%s\n" % exe_string)
    print_mutex.release()

    #os.system(exe_string)
    subprocess.call(exe_string, shell=True, executable=SHELLPATH)


class Tool(AlignmentRoutines):

    def __init__(self, cmd, path="", max_threads=4, jar_path="", jar=None,
                 max_memory="500m", max_per_thread_memory="500m", timelog=None, tmp_dir=None):
        SequenceRoutines.__init__(self)
        self.path = self.check_path(path)
        self.cmd = cmd
        self.threads = max_threads
        #print(jar_path)
        self.jar_path = self.check_path(jar_path) if jar_path else ""
        self.jar = jar
        self.max_memory = max_memory
        self.timelog = timelog
        self.max_per_thread_memory = max_per_thread_memory
        self.tmp_dir = tmp_dir

    def execute(self, options="", cmd=None, directory=None, capture_output=False, generate_cmd_string_only=False, intepreter=None):
        command = cmd if cmd is not None else self.cmd

        exe_string = ""
        exe_string += " cd %s && " % self.check_dir_path(directory) if directory else ""
        exe_string += ("%s " % intepreter if intepreter else "") + (self.check_path(self.path) if self.path else "") + command + " " + options

        sys.stdout.write("Executing:\n\t%s\n" % exe_string)
        if self.timelog:
            os.system("date >> %s" % self.timelog)
            with open(self.timelog, "a") as time_fd:
                time_fd.write("Command\t%s\n" % exe_string)

        if generate_cmd_string_only:
            return exe_string

        if capture_output:
            return Popen([exe_string], shell=True, stdout=PIPE).stdout  # returns file object
        else:
            subprocess.call(exe_string, shell=True, executable=SHELLPATH)
            if self.timelog:
                os.system("date >> %s" % self.timelog)
            return None

    def parallel_execute(self, options_list, cmd=None, capture_output=False, threads=None, dir_list=None,
                         write_output_to_file=None, external_process_pool=None, async_run=False,
                         job_chunksize=1, output_file_list=None, duplicate_to_stdout=False, intepreter=None):

        command = cmd if cmd is not None else self.cmd
        number_of_commands = len(options_list)

        if dir_list:
            if isinstance(dir_list, str):
                directory_list = [dir_list for i in range(0, number_of_commands)]
            elif len(dir_list) == number_of_commands:
                directory_list = [directory for directory in dir_list]
            else:
                raise ValueError("Error during option parsing for parallel execution in different folders. "
                                 "Length of directory list is not equal to length of option list")
        else:
            directory_list = [None for i in range(0, number_of_commands)]

        if output_file_list:
            if len(dir_list) == number_of_commands:
                out_list = [out_file for out_file in output_file_list]
            else:
                raise ValueError("Error during option parsing for parallel execution with storing stdout and stderr. "
                                 "Length of output file list is not equal to length of option list")
        else:
            out_list = [None for i in range(0, number_of_commands)]

        exe_string_list = []

        for options, directory, output in zip(options_list, directory_list, out_list):
            com = ""
            com += " cd %s && " % self.check_dir_path(directory) if directory else ""
            com += ("%s " % intepreter if intepreter else "") + " %s%s" % (self.check_path(self.path) if self.path else "", command)
            com += " %s" % options

            if output:
                com += " | tee %s/%s" % (directory, output) if duplicate_to_stdout else " > %s/%s 2>&1" % (directory, output)

            exe_string_list.append(com)

        with open("exe_list.t", "a") as exe_fd:
            for entry in exe_string_list:
                exe_fd.write("%s\n" % entry)
        process_pool = external_process_pool if external_process_pool else mp.Pool(threads if threads else self.threads)

        results = process_pool.map_async(execute, exe_string_list, chunksize=job_chunksize) if async_run else process_pool.map(execute, exe_string_list, chunksize=job_chunksize)
        #process_pool.close()
        if write_output_to_file:
            with open(write_output_to_file, "w") as out_fd:
                out_fd.write(results)
        return results if capture_output else None

    def generate_slurm_job_script(self,
                                  job_name,
                                  log_prefix,
                                  task_commands,
                                  error_log_prefix,
                                  node_number=None,
                                  partition=None,
                                  qos=None,
                                  cpus_per_node=None,
                                  node_type=None,
                                  max_memory_per_node=None,
                                  stdout_file=None,
                                  email=None,
                                  mail_type=None,
                                  job_script=None,
                                  task_index_list=None,
                                  start_task_index=None,
                                  end_task_index=None,
                                  max_running_jobs=None,
                                  max_running_time=None,
                                  cpus_per_task=None,
                                  max_memmory_per_cpu=None,
                                  modules_list=None,
                                  environment_variables_dict=None):

        #if (not task_index_list) and ((not start_task_index) or (not end_task_index)):
        #    raise ValueError("Neither task index list nor start or end task index were set")

        script = "#!/usr/bin/env bash\n"
        if task_index_list or start_task_index or end_task_index:
            script += "#SBATCH --array=%s" % (",".join(map(str, task_index_list)) if task_index_list else "%s-%s" % (str(start_task_index),
                                                                                                                     str(end_task_index)))
            script += "%s\n" % ("%%%i" % max_running_jobs if max_running_jobs else "")

        script += "#SBATCH --time=%s         # Run time in hh:mm:ss\n" % max_running_time if max_running_time else ""
        script += "#SBATCH --mem-per-cpu=%i       # Minimum memory required per CPU (in megabytes)\n" % max_memmory_per_cpu if max_memmory_per_cpu else ""
        script += "#SBATCH --cpus-per-task=%i\n" % cpus_per_task if cpus_per_task else ""
        script += "#SBATCH --job-name=%s\n" % job_name if job_name else ""
        script += "#SBATCH --error=%s.%%A_%%a.err\n" % error_log_prefix
        script += "#SBATCH --output=%s.%%A_%%a.log\n" % log_prefix

        script += "#SBATCH --nodes=%i\n" % node_number if node_number else ""
        script += "#SBATCH --partition=%s\n" % partition if partition else ""
        script += "#SBATCH --ntasks-per-nod=%i\n" % cpus_per_node if cpus_per_node else ""

        script += "#SBATCH --node_type=%s\n" % node_type if node_type else ""

        script += "#SBATCH --mem=%s\n" % str(max_memory_per_node) if max_memory_per_node else ""
        script += "#SBATCH --mail-user=%s\n" % email if email else ""
        script += "#SBATCH --mail-type=%s\n" % mail_type if mail_type else ""
        script += "#SBATCH --output=%s\n" % stdout_file if stdout_file else ""

        script += "\n"

        if environment_variables_dict:
            script += "#--------------Environment variables--------------\n\n"
            for variable in environment_variables_dict:
                script += "%s=%s\n" % (variable, environment_variables_dict[variable])
            script += "#-------------------------------------------\n\n"

        if modules_list:
            script += "#--------------Modules--------------\n\n"
            modules_to_load = [modules_list] if isinstance(modules_list, str) else list(modules_list)
            for module in modules_to_load:
                script += "module load %s\n" % module

            script += "#-------------------------------------------\n\n"

        script += "#--------------Commands--------------\n\n"

        script += "%s\n" % task_commands

        script += "#-------------------------------------------\n\n"

        if job_script:
            with open(job_script, "w") as job_fd:
                job_fd.write(script)

        return script

    def slurm_run_job_from_script(self,
                                  job_array_script,
                                  after_job_id_list=[],
                                  afterany_job_id_list=[],
                                  afternotok_job_id_list=[],
                                  afterok_job_id_list=[],
                                  singleton=False):
        """
        after:jobid[:jobid...]	    job can begin after the specified jobs have started
        afterany:jobid[:jobid...]	job can begin after the specified jobs have terminated
        afternotok:jobid[:jobid...]	job can begin after the specified jobs have failed
        afterok:jobid[:jobid...]	job can begin after the specified jobs have run to completion with an exit code of zero (see the user guide for caveats).
        singleton	                jobs can begin execution after all previously launched jobs with the same name and user have ended. This is useful to collate results of a swarm or to send a notification at the end of a swarm.

        """

        dependencies_option_list = []

        if after_job_id_list:
            dependencies_option_list.append("after:%s" % (after_job_id_list if isinstance(after_job_id_list, str) else ":".join(after_job_id_list)))
        if afterany_job_id_list:
            dependencies_option_list.append("afterany:%s" % (afterany_job_id_list if isinstance(afterany_job_id_list, str) else ":".join(afterany_job_id_list)))
        if afternotok_job_id_list:
            dependencies_option_list.append("afternotok:%s" % (afternotok_job_id_list if isinstance(afternotok_job_id_list, str) else ":".join(afternotok_job_id_list)))
        if afterok_job_id_list:
            #print afterok_job_id_list
            dependencies_option_list.append("afterok:%s" % (afterok_job_id_list if isinstance(afterok_job_id_list, str) else ":".join(afterok_job_id_list)))
        if singleton:
            dependencies_option_list.append("singleton")

        options = ""

        if dependencies_option_list:
            dependencies_option = ",".join(dependencies_option_list)
            options += " --dependency=%s" % dependencies_option

        options += " %s" % job_array_script

        command = "sbatch %s " % options

        # Popen.stdout returns file object
        print(command)
        job_id = Popen([command], shell=True, stdout=PIPE, executable=SHELLPATH).stdout.readline().strip().split()[-1]
        #print "Job id", job_id
        return job_id

    def slurm_run_job(self,
                      job_name,
                      log_prefix,
                      task_commands,
                      error_log_prefix,
                      job_script,
                      node_number=None,
                      partition=None,
                      qos=None,
                      cpus_per_node=None,
                      node_type=None,
                      max_memory_per_node=None,
                      stdout_file=None,
                      email=None,
                      mail_type=None,
                      task_index_list=None,
                      start_task_index=None,
                      end_task_index=None,
                      max_running_jobs=None,
                      max_running_time=None,
                      cpus_per_task=None,
                      max_memmory_per_cpu=None,
                      modules_list=None,
                      environment_variables_dict=None,
                      after_job_id_list=[],
                      afterany_job_id_list=[],
                      afternotok_job_id_list=[],
                      afterok_job_id_list=[],
                      singleton=False):

        self.generate_slurm_job_script(job_name,
                                       log_prefix,
                                       task_commands,
                                       error_log_prefix,
                                       node_number=node_number,
                                       partition=partition,
                                       qos=qos,
                                       cpus_per_node=cpus_per_node,
                                       node_type=node_type,
                                       max_memory_per_node=max_memory_per_node,
                                       stdout_file=stdout_file,
                                       email=email,
                                       mail_type=mail_type,
                                       job_script=job_script,
                                       task_index_list=task_index_list,
                                       start_task_index=start_task_index,
                                       end_task_index=end_task_index,
                                       max_running_jobs=max_running_jobs,
                                       max_running_time=max_running_time,
                                       cpus_per_task=cpus_per_task,
                                       max_memmory_per_cpu=max_memmory_per_cpu,
                                       modules_list=modules_list,
                                       environment_variables_dict=environment_variables_dict)

        return self.slurm_run_job_from_script(job_script,
                                              after_job_id_list=after_job_id_list,
                                              afterany_job_id_list=afterany_job_id_list,
                                              afternotok_job_id_list=afternotok_job_id_list,
                                              afterok_job_id_list=afterok_job_id_list,
                                              singleton=singleton)

    def parse_sbatch_options(self,
                             job_name=None,
                             log_prefix=None,
                             error_log_prefix=None,
                             node_number=None,
                             partition=None,
                             cpus_per_node=None,
                             node_type=None,
                             max_memory_per_node=None,
                             stdout_file=None,
                             email=None,
                             mail_type=None,
                             task_index_list=None,
                             start_task_index=None,
                             end_task_index=None,
                             max_running_jobs=None,
                             max_running_time=None,
                             cpus_per_task=None,
                             max_memmory_per_cpu=None):

        options = ""
        if task_index_list or start_task_index or end_task_index:
            options += " --array=%s" % (",".join(map(str, task_index_list)) if task_index_list else "%s-%s" % (str(start_task_index),
                                                                                                                     str(end_task_index)))
            options += "%s" % ("%%%i" % max_running_jobs if max_running_jobs else "")

        options += " --time=%s" % max_running_time if max_running_time else ""
        options += " --mem-per-cpu=%i" % max_memmory_per_cpu if max_memmory_per_cpu else ""
        options += " --cpus-per-task=%i" % cpus_per_task if cpus_per_task else ""
        options += " --job-name=%s" % job_name if job_name else ""
        options += " --error=%s.%%A_%%a.err" % error_log_prefix if error_log_prefix else ""
        options += " --output=%s.%%A_%%a.log" % log_prefix if log_prefix else ""

        options += " --nodes=%i" % node_number if node_number else ""
        options += " --partition=%s" % partition if partition else ""
        options += " --ntasks-per-nod=%i" % cpus_per_node if cpus_per_node else ""

        options += " --node_type=%s" % node_type if node_type else ""

        options += " --mem=%s" % str(max_memory_per_node) if max_memory_per_node else ""
        options += " --mail-user=%s" % email if email else ""
        options += " --mail-type=%s" % mail_type if mail_type else ""
        options += " --output=%s" % stdout_file if stdout_file else ""

        return options

    def slurm_run_multiple_jobs_in_wrap_mode(self,
                                             cmd_list,
                                             cmd_log,
                                             max_jobs=None,
                                             job_name=None,
                                             log_prefix=None,
                                             error_log_prefix=None,
                                             node_number=None,
                                             partition=None,
                                             cpus_per_node=None,
                                             node_type=None,
                                             max_memory_per_node=None,
                                             stdout_file=None,
                                             email=None,
                                             mail_type=None,
                                             task_index_list=None,
                                             start_task_index=None,
                                             end_task_index=None,
                                             max_running_jobs=None,
                                             max_running_time=None,
                                             cpus_per_task=None,
                                             max_memmory_per_cpu=None,
                                             max_memory_per_cpu_per_task_list=None,
                                             modules_list=None,
                                             environment_variables_dict=None,
                                             verbose=False):
        if max_memory_per_cpu_per_task_list is not None:
            if len(max_memory_per_cpu_per_task_list) != len(cmd_list):
                raise ValueError("Length of cmd_list(%i) is noq equal to max_memory_per_cpu_per_task_list(%i)" % (len(cmd_list), len(max_memory_per_cpu_per_task_list)))

        with open(cmd_log, "w") as cmd_fd:
            cmd_fd.write("#job_id\tcmd_ids\tsbatch_cmd\n")
            environment_variables_cmd = ""
            modules_cmd = ""

            if environment_variables_dict:
                for variable in environment_variables_dict:
                    environment_variables_cmd += " %s=%s; " % (variable, environment_variables_dict[variable])

            if modules_list:
                modules_to_load = [modules_list] if isinstance(modules_list, str) else list(modules_list)
                for module in modules_to_load:
                    modules_cmd += " module load %s; " % module

            sbatch_options = self.parse_sbatch_options(job_name=job_name,
                                                       log_prefix=log_prefix,
                                                       error_log_prefix=error_log_prefix,
                                                       node_number=node_number,
                                                       partition=partition,
                                                       cpus_per_node=cpus_per_node,
                                                       node_type=node_type,
                                                       max_memory_per_node=max_memory_per_node,
                                                       stdout_file=stdout_file,
                                                       email=email,
                                                       mail_type=mail_type,
                                                       task_index_list=task_index_list,
                                                       start_task_index=start_task_index,
                                                       end_task_index=end_task_index,
                                                       max_running_jobs=max_running_jobs,
                                                       max_running_time=max_running_time,
                                                       cpus_per_task=cpus_per_task,
                                                       max_memmory_per_cpu=max_memmory_per_cpu)

            job_submit_index_array = np.linspace(0, len(cmd_list), max_jobs, dtype=int) if max_jobs and (max_jobs < len(cmd_list)) else [i for i in range(0, len(cmd_list) + 1)]

            for job_index in range(0, len(job_submit_index_array) - 1):

                if max_memory_per_cpu_per_task_list:
                    #print "----------------------------"
                    #print max_memory_per_cpu_per_task_list
                    #print job_submit_index_array
                    #print job_index
                    #print "============================"
                    max_memory_per_cpu_for_job = max(max_memory_per_cpu_per_task_list[job_submit_index_array[job_index]:job_submit_index_array[job_index+1]])

                    sbatch_options = self.parse_sbatch_options(job_name=job_name,
                                                               log_prefix=log_prefix,
                                                               error_log_prefix=error_log_prefix,
                                                               node_number=node_number,
                                                               partition=partition,
                                                               cpus_per_node=cpus_per_node,
                                                               node_type=node_type,
                                                               max_memory_per_node=max_memory_per_node,
                                                               stdout_file=stdout_file,
                                                               email=email,
                                                               mail_type=mail_type,
                                                               task_index_list=task_index_list,
                                                               start_task_index=start_task_index,
                                                               end_task_index=end_task_index,
                                                               max_running_jobs=max_running_jobs,
                                                               max_running_time=max_running_time,
                                                               cpus_per_task=cpus_per_task,
                                                               max_memmory_per_cpu=max_memory_per_cpu_for_job)

                options = "%s" % sbatch_options
                options += " --wrap \"%s %s %s\"" % (environment_variables_cmd,
                                                     modules_cmd,
                                                     ";".join(cmd_list[job_submit_index_array[job_index]:job_submit_index_array[job_index+1]]))
                command = "sbatch %s" % options

                if verbose:
                    print(command)

                job_id = Popen([command], shell=True, stdout=PIPE).stdout.readline().strip().split()[-1]
                cmd_fd.write("%s\t%i-%i\t%s\n" % (job_id,
                                                  job_submit_index_array[job_index],
                                                  job_submit_index_array[job_index+1] - 1, command))


class JavaTool(Tool):

    def __init__(self, jar, java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):

        Tool.__init__(self, "java", path=java_path, max_threads=max_threads,
                      jar_path=jar_path, jar=jar, max_memory=max_memory, timelog=timelog, tmp_dir=None)

    def execute(self, options="", cmd=None, capture_output=False, runtype="jar", generate_cmd_string_only=False):
        command = cmd if cmd is not None else ""

        java_string = "java"
        java_string += " -Xmx%s" % str(self.max_memory) if self.max_memory else ""
        java_string += " -Djava.io.tmpdir=%s" % self.tmp_dir if self.tmp_dir else ""
        #print (self.jar_path)
        java_string += " -%s %s%s" % (runtype, self.check_dir_path(self.jar_path) if self.jar_path else "", self.jar)
        java_string += " %s" % command
        java_string += " %s" % options

        exe_string = (self.check_path(self.path) if self.path else "") + java_string

        sys.stdout.write("Executing:\n\t%s\n" % exe_string)
        if self.timelog:
            os.system("date >> %s" % self.timelog)
            with open(self.timelog, "a") as time_fd:
                time_fd.write("Command\t%s\n" % exe_string)

        if generate_cmd_string_only:
            return exe_string

        if capture_output:
            return Popen([exe_string], shell=True, stdout=PIPE, executable=SHELLPATH).stdout  # returns file object
        else:
            subprocess.call(exe_string, shell=True, executable=SHELLPATH)
            if self.timelog:
                os.system("date >> %s" % self.timelog)
            return None

    def parallel_execute(self, options_list, cmd=None, capture_output=False, threads=None, dir_list=None,
                         write_output_to_file=None, external_process_pool=None, async_run=False, runtype="jar",
                         job_chunksize=1):
        command = cmd if cmd is not None else self.cmd
        if dir_list:
            if isinstance(dir_list, str):
                exe_string_list = [("cd %s && " % dir_list) + (self.check_path(self.path) if self.path else "")
                                   + command + " -%s %s/%s" % (runtype, self.check_dir_path(self.jar_path) if self.jar_path else "", self.jar)
                                   + " " + options for options in options_list]

            elif isinstance(dir_list, Iterable) and (len(options_list) == len(dir_list)):
                exe_string_list = [("cd %s && " % directory) + (self.check_path(self.path) if self.path else "")
                                   + command + " -%s %s/%s" % (runtype, self.check_dir_path(self.jar_path) if self.jar_path else "", self.jar)
                                   + " " + options for options, directory in zip(options_list, dir_list)]
            else:
                raise ValueError("Error during option parsing for parallel execution in different folders. "
                                 "Length of directory list is not equal to length of option list")

        else:
            exe_string_list = [(self.check_path(self.path) if self.path else "")
                               + command + " -%s %s/%s" % (runtype, self.check_dir_path(self.jar_path) if self.jar_path else "", self.jar)
                               + " " + options for options in options_list]

        with open("exe_list.t", "a") as exe_fd:
            for entry in exe_string_list:
                exe_fd.write("%s\n" % entry)
        process_pool = external_process_pool if external_process_pool else mp.Pool(threads if threads else self.threads)

        results = process_pool.map_async(execute, exe_string_list, chunksize=job_chunksize) if async_run else process_pool.map(execute, exe_string_list, chunksize=job_chunksize)
        #process_pool.close()
        if write_output_to_file:
            with open(write_output_to_file, "w") as out_fd:
                out_fd.write(results)
        return results if capture_output else None


#!/usr/bin/env python

import sys
from collections import OrderedDict

from ete2 import Tree #, TreeStyle, AttrFace, faces, NodeStyle
from RouToolPa.Collections.General import IdList, TwoLvlDict, SynDict


class CodeMLReport():
    """
    Only parsing  of  dN, dS and omega trees is supported
    """
    def __init__(self, reportfile, treefile=None):
        self.dStree = None
        self.dNtree = None
        self.Wtree = None
        self.LnL = None
        with open(reportfile, "r") as in_fd:
            for line in in_fd:
                if line[:8] == "dS tree:":
                    self.dStree = Tree(in_fd.next())
                    continue
                elif line[:8] == "dN tree:":
                    self.dNtree = Tree(in_fd.next())
                    continue
                elif line.strip() == "w ratios as labels for TreeView:":
                    self.Wtree = Tree(in_fd.next().replace(" #", ": "))  # convert omega tree to readable by ete2 format
                    continue
                elif line[:10] == "lnL(ntime:":
                    self.LnL = float(line.strip().split(":")[-1].split()[0])
                    continue
        if treefile:
            with open(treefile, "r") as tree_fd:
                self.tree = Tree(tree_fd.readline())
            i = 1
            if self.dStree and self.dNtree and self.Wtree:
                for tree_node, dNnode, dSnode, Wnode in zip(self.tree.traverse(), self.dNtree.traverse(), self.dStree.traverse(), self.Wtree.traverse()):
                    tree_node.add_feature("dN", dNnode.dist)
                    tree_node.add_feature("dS", dSnode.dist)
                    tree_node.add_feature("W", Wnode.dist)
                    tree_node.add_feature("id", i)
                    i += 1
        else:
            self.tree = None

    def write_trees(self, treefile_prefix, format=5):
        dNtree_file = "%s_dN.nwk" % treefile_prefix
        dStree_file = "%s_dS.nwk" % treefile_prefix
        Wtree_file = "%s_W.nwk" % treefile_prefix
        tree_file = "%s_all.nwk" % treefile_prefix
        with open(dNtree_file, "w") as dn_fd:
            dn_fd.write(self.dNtree.write(format=format) + "\n")
        with open(dStree_file, "w") as ds_fd:
            ds_fd.write(self.dStree.write(format=format) + "\n")
        with open(Wtree_file, "w") as w_fd:
            w_fd.write(self.Wtree.write(format=format) + "\n")
        if self.tree:
            with open(tree_file, "w") as t_fd:
                t_fd.write(self.tree.write(format=format, features={"dN", "dS", "W"}) + "\n")

    def get_all_values(self, output_file):
        with open(output_file, "w") as out_fd:
            out_fd.write("#dN\tdS\tW\n")
            for dNnode, dSnode, Wnode in zip(self.dNtree.traverse(), self.dStree.traverse(), self.Wtree.traverse()):
                if dNnode.is_root():
                    continue
                out_fd.write("%f\t%f\t%f\n" % (dNnode.dist, dSnode.dist, Wnode.dist))

    @staticmethod
    def _get_tree_dist_values(tree, expression=None):
        feature_values_list = IdList()
        for node in tree.traverse():
            if expression is not None:
                if not expression(node):
                    continue
            feature_values_list.append(node.dist)
        return feature_values_list

    @staticmethod
    def _get_tree_dist_dict(tree):
        feature_values_dict = OrderedDict()
        for node in tree.traverse():
            if not node.is_leaf():
                continue
            feature_values_dict[node.name] = node.dist
        return feature_values_dict

    def get_leaf_values(self, write=True):
        leaf_values_dict = TwoLvlDict()
        dN_dict = self._get_tree_dist_dict(self.dNtree)
        dS_dict = self._get_tree_dist_dict(self.dStree)
        W_fict = self._get_tree_dist_dict(self.Wtree)

        leaf_values_dict["dN"] = dN_dict
        leaf_values_dict["dS"] = dS_dict
        leaf_values_dict["W"] = W_fict

        if write:
            leaf_values_dict.write("leaf_values.t")
        return leaf_values_dict

    def find_leaves_with_positive_selection(self, write=True):
        leaf_values_dict = self.get_leaf_values(write=False)
        positive_selected_leaves_dict = SynDict()
        for leaf_name in leaf_values_dict["W"]:
            if leaf_values_dict["W"][leaf_name] > 1:
                positive_selected_leaves_dict[leaf_name] = leaf_values_dict["W"][leaf_name]
        if write:
            positive_selected_leaves_dict.write("leaves_with_positive_selection.t")
        return positive_selected_leaves_dict

    def get_feature_values(self, mode="all"):
        if mode == "all":
            expression = lambda node: not node.is_root()
            suffix = "_all.t"
        elif mode == "leaves":
            expression = lambda node: node.is_leaf() and (not node.is_root())
            suffix = "_leaves.t"
        elif mode == "internal":
            expression = lambda node: not node.is_leaf() and (not node.is_root())
            suffix = "_internal.t"
        else:
            raise ValueError("Possible modes: all, leaves, internal")

        dN_list = self._get_tree_dist_values(self.dNtree, expression=expression)
        dS_list = self._get_tree_dist_values(self.dStree, expression=expression)
        W_list = self._get_tree_dist_values(self.Wtree, expression=expression)

        dN_list.write("dN%s" % suffix)
        dS_list.write("dS%s" % suffix)
        W_list.write("W%s" % suffix)

        return dN_list, dS_list, W_list

    """
    def draw_trees(self, output_prefix, pic_width=900):
        tree = self.tree
        print self.tree
        print tree.write(format=5, features={"dN", "dS", "W"})
        red_style = NodeStyle()
        red_style["fgcolor"] = "#0f0f0f"
        red_style["size"] = 0
        red_style["vt_line_color"] = "#ff0000"
        red_style["hz_line_color"] = "#ff0000"
        red_style["vt_line_width"] = 8
        red_style["hz_line_width"] = 8

        for node in tree.traverse():
            node.img_style["size"] = 0
            node.dS = "%.2f" % node.dS
            node.dN = "%.2f" % node.dN
            node.dW = "%.2f" % node.dW
            #if node.W > 1.0:
            #    node.set_style(red_style)

        def layout(node):
            if node.up is not None:
                attr = AttrFace("dN", fsize=4, fgcolor="green", text_prefix="")
                faces.add_face_to_node(attr, node, 0, position="branch-top")
                attr = AttrFace("dS", fsize=4, fgcolor="red", text_prefix="")
                faces.add_face_to_node(attr, node, 0, position="branch-bottom")
            if node.is_leaf():
                attr = AttrFace("name", fsize=10, fgcolor="black", text_prefix="  ", fstyle="italic")
                faces.add_face_to_node(attr, node, 0, position="aligned")

        def omega_layout(node):
            if node.up is not None:
                attr = AttrFace("W", fsize=4, fgcolor="black", text_prefix="")
                faces.add_face_to_node(attr, node, 0, position="branch-top")

            if node.is_leaf():
                attr = AttrFace("name", fsize=10, fgcolor="black", text_prefix="  ", fstyle="italic")
                faces.add_face_to_node(attr, node, 0, position="aligned")

        ts = TreeStyle()
        ts.layout_fn = layout
        ts.optimal_scale_level = "full"
        ts.branch_vertical_margin = 10
        ts.show_leaf_name = False
        tree.render("%s.png" % output_prefix)
        tree.render("%s_dN_dS_tree.png" % output_prefix, w=pic_width, units='mm', tree_style=ts, dpi=200)

        omega_ts = TreeStyle()
        omega_ts.layout_fn = omega_layout
        omega_ts.optimal_scale_level = "full"
        omega_ts.branch_vertical_margin = 10
        omega_ts.show_leaf_name = False

        tree.render("%s_W_tree.png" % output_prefix, w=pic_width, units='mm', tree_style=omega_ts, dpi=300)
    """
    def branches_with_positive_selection(self):
        for node in self.Wtree.traverse():
            if node.dist > 1.0:
                return True
        return False

    def convert_trees_to_tsv(self, output_prefix):

        w_tab_file = "%s.dn.ds.w.tab" % output_prefix
        id_tree_file = "%s.id.nwk" % output_prefix
        with open(w_tab_file, 'w') as w_fd:
            w_fd.write("#node_id\tnode_name\tdN\tdS\tW\n")
            if self.tree:
                for node in self.tree.traverse():
                    w_fd.write("%i\t%s\t%f\t%f\t%f\n" % (node.id,
                                                         node.name if node.name else ".",
                                                         node.dN,
                                                         node.dS,
                                                         node.W))
                    with open(id_tree_file, "w") as t_fd:
                        t_fd.write(self.tree.write(format=5, features={"dN", "dS", "W", "id"}) + "\n")
            else:
                i = 1
                for dNnode, dSnode, Wnode in zip(self.dNtree.traverse(), self.dStree.traverse(), self.Wtree.traverse()):

                    for node in dNnode, dSnode, Wnode:
                        node.add_feature("id", i)

                    w_fd.write("%i\t%s\t%f\t%f\t%f\n" % (node.id,
                                                         node.name if node.name else ".",
                                                         dNnode.dist,
                                                         dSnode.dist,
                                                         Wnode.dist))
                    i += 1

                    with open(id_tree_file, "w") as t_fd:
                        t_fd.write(self.dNtree.write(format=format, features={"id"}) + "\n")

    def extract_trees(self, out_prefix):
        self.write_trees(out_prefix)
        self.get_feature_values(mode="all")
        self.get_feature_values(mode="leaves")
        self.get_feature_values(mode="internal")
        self.get_all_values("dN_dS_W.t")
        self.get_leaf_values()
        if self.branches_with_positive_selection():
            sys.stderr.write("Presence of branches with positive selection\n")

        self.convert_trees_to_tsv(out_prefix)




#!/usr/bin/env python
__author__ = 'mahajrod'
import numpy as np
from collections import OrderedDict

from RouToolPa.Routines import MatplotlibRoutines

from RouToolPa.Collections.General import TwoLvlDict


class FaCutReport:
    def __init__(self, facut_report_file):
        self.table_ids = OrderedDict({"instrument_id": 0,
                                      "run_number": 1,
                                      "flowcell_id": 2,
                                      "lane_number": 3,
                                      "tile": 4,
                                      "both_retained": 5,
                                      "forward_only": 6,
                                      "reverse_only": 7,
                                      "both_discarded": 8,
                                      "total": 9})
        self.tile_table = []
        self.machine_id_list = []
        self.flowcell_id_list = []

        with open(facut_report_file, "r") as in_fd:
            tmp = ["", ""]
            while tmp[0] != "Paires retained:":
                tmp = in_fd.readline().strip().split("\t")

            self.retained_pairs = int(tmp[1])

            self.retained_forward_only = int(in_fd.readline().strip().split("\t")[1])
            self.retained_reverse_only = int(in_fd.readline().strip().split("\t")[1])
            self.both_discarded = int(in_fd.readline().strip().split("\t")[1])
            self.input_pairs = self.retained_pairs + self.retained_forward_only + self.retained_reverse_only + self.both_discarded
            self.retained_pairs_fraction = float(self.retained_pairs) / float(self.input_pairs)

            tmp = "aaaaaaaaaaaaaaaaabbbbbbbbbbbbb"
            while tmp[:13] != "instrument_id":
                tmp = in_fd.readline()

            for line in in_fd:
                tmp = line.strip().split("\t")
                for i in (1, 3, 4, 5, 6, 7, 8):
                    tmp[i] = int(tmp[i])
                tmp.append(tmp[5] + tmp[6] + tmp[7] + tmp[8])
                if tmp[0] not in self.machine_id_list:
                    self.machine_id_list.append(tmp[0])
                tmp[0] = self.machine_id_list.index(tmp[0])  # replace machine id by index

                if tmp[2] not in self.flowcell_id_list:
                    self.flowcell_id_list.append(tmp[2])
                tmp[2] = self.flowcell_id_list.index(tmp[2])  # replace flowcell id by index

                self.tile_table.append(tmp)
        #print self.tile_table
        self.tile_table = np.array(self.tile_table)

        self.minimum_retained_pairs_in_tiles_fraction = min(self.get_fraction_of_retained_pairs_per_tile())

        self.lane_table = self.get_lane_table()
        self.full_lane_id_list = self.get_full_lane_id_list()
        self.short_lane_id_list = self.get_short_lane_id_list() # WARNING! only lane number is stored here, so if there are lanes with same numbers of several runs they will collaps

        self.more_than_one_machine = True if len(self.machine_id_list) > 1 else False
        self.more_than_one_flowcell = True if len(self.flowcell_id_list) > 1 else False
        self.more_than_one_lane = True if len(self.lane_table) > 1 else False

    def get_fraction_of_retained_pairs_per_tile(self):
        return np.array(self.tile_table[:, 5], dtype=float) / np.array(self.tile_table[:, 9], dtype=float)

    def draw_fraction_of_retained_pairs_per_tile_histogram(self, output_prefix):
        data = self.get_fraction_of_retained_pairs_per_tile()
        MatplotlibRoutines.percent_histogram(data, output_prefix,
                                             n_bins=20, title="Distribution of retained pairs per tile",
                                             xlabel="Fraction of retained pairs", ylabel="Number of tiles", label=None,
                                             extensions=("png", "svg"), legend=None, legend_location="best",
                                             input_mode="fraction", xmax=None, xmin=None)

    def get_lane_table(self):
        # split tile_table by lanes
        lane_tile_list = np.split(self.tile_table,
                                  np.unique(np.where(np.diff(self.tile_table[:, 0:self.table_ids["tile"]], axis=0))[0]) + 1)
        """
        print len((self.tile_table[:, 0:self.table_ids["tile"]]))
        print list(np.where(np.diff(self.tile_table[:, 0:self.table_ids["tile"]], axis=0)))
        print "CCCCCCCCCCCCCCCCCCCCCC"

        print list(np.diff(self.tile_table[:, 0:self.table_ids["tile"]], axis=0))
        print "BBBBBBBBBBBBBBB"


        print lane_tile_list
        print "AAAAAAAAAAAAAAAAAAAAAAAAAAa"
        print "\n"
        """
        #print lane_tile_list
        lane_table = []
        for lane_tile_table in lane_tile_list:
            #print lane_tile_table
            #print "\n"
            lane_ids = list(lane_tile_table[0][:self.table_ids["tile"]])
            lane_values = list(np.sum(lane_tile_table[:, self.table_ids["both_retained"]:], axis=0))

            lane_table.append(lane_ids + [0] + lane_values)

        return np.array(lane_table)

    def lane_table_string(self):
        lane_str = "\t".join(self.table_ids.keys()) + "\n"
        for lane_line in self.lane_table:
            tmp = list(lane_line)
            tmp[0] = self.machine_id_list[tmp[0]]
            tmp[2] = self.flowcell_id_list[tmp[2]]
            lane_str += "\t".join(map(str, tmp)) + "\n"

        return lane_str

    def get_full_lane_id_list(self):
        lane_id_list = []
        for lane_line in self.lane_table:
            tmp = list(lane_line)
            lane_id_list.append(":".join([self.machine_id_list[tmp[0]],
                                          str(tmp[1]),
                                          self.flowcell_id_list[tmp[2]],
                                          str(tmp[3])]))
        return lane_id_list

    def get_short_lane_id_list(self):
        return list(np.unique(self.tile_table[:, self.table_ids["lane_number"]]))


class FaCutReportCollection(OrderedDict):
    def __init__(self, report_dict=None, file_dict=None):
        OrderedDict.__init__(self)
        if report_dict:
            for report_id in report_dict:
                self[report_id] = report_dict
        if file_dict:
            for file_id in file_dict:
                if file_id in self:
                    raise KeyError("Report with same id(%s) is already present in FaCutReportCollection" % str(file_id))
                self[file_id] = FaCutReport(file_dict[file_id])

    def get_general_stats(self):
        stat_dict = TwoLvlDict()

        for report_id in self:
            stat_dict[report_id] = OrderedDict()
            stat_dict[report_id]["machine_number"] = len(self[report_id].machine_id_list)
            stat_dict[report_id]["machine_ids"] = self[report_id].machine_id_list
            stat_dict[report_id]["flowcell_number"] = len(self[report_id].flowcell_id_list)
            stat_dict[report_id]["flowcell_ids"] = self[report_id].flowcell_id_list
            stat_dict[report_id]["lane_number"] = len(self[report_id].lane_table)
            stat_dict[report_id]["full_lane_ids"] = self[report_id].full_lane_id_list
            stat_dict[report_id]["short_lane_ids"] = self[report_id].short_lane_id_list
            stat_dict[report_id]["input_pairs"] = self[report_id].input_pairs
            stat_dict[report_id]["retained_pairs"] = self[report_id].retained_pairs
            stat_dict[report_id]["retained_pairs_fraction"] = self[report_id].retained_pairs_fraction
            stat_dict[report_id]["retained_forward_only"] = self[report_id].retained_forward_only
            stat_dict[report_id]["retained_reverse_only"] = self[report_id].retained_reverse_only
            stat_dict[report_id]["both_discarded"] = self[report_id].both_discarded
            stat_dict[report_id]["min_retained_pairs_in_tiles_fraction"] = self[report_id].minimum_retained_pairs_in_tiles_fraction

        return stat_dict

    def write_general_stats(self, output_file):
        stat_dict = self.get_general_stats()
        stat_dict.write(output_file, absent_symbol=".", sort=False)



#!/usr/bin/env python

from collections import OrderedDict

VCF_COLS = OrderedDict({"CHROM":  0,
                        "POS":    1,
                        "ID":     2,
                        "REF":    3,
                        "ALT":    4,
                        "QUAL":   5,
                        "FILTER": 6,
                        "INFO":   7,
                        "FORMAT": 8})



__author__ = 'mahajrod'

import numpy as np


def parse_R_table_file(table_file, separator=","):
    with open(table_file, "r") as fd_table:
        column_names = [name[1:-1]for name in fd_table.readline().strip().split(separator)[1:]]
        row_names = []
        table = []
        for line in fd_table:
            if line == "\n":
                break
            temp_line = line.strip().split(separator)
            row_names.append(temp_line[0][1:-1])
            table.append([float(x) for x in temp_line[1:]])
    return column_names, row_names, np.array(table)


def get_indices_from_names(names_list, queue_names_list):
    result = []
    for entry in queue_names_list:
        result.append(names_list.index(entry))
    return sorted(result)


#def get_submatrix_by_column(table, column_list, row_list):
#    return table[row_list][column_list]


if __name__ == "__main__":
    column_names, row_names, table = parse_R_table_file("/home/mahajrod/genetics/MH_selection/trees/all_species/tree_dist.csv", separator=",")
    print(column_names)
    print(row_names)
    print(np.shape(table))
    indices = get_indices_from_names(row_names, ['AY442348.1', 'NC_001778.1', 'NC_020655.1', 'HM143934.1'])
    #print(indices)
    extracted_table = table[np.array(indices), :][:, np.array(indices)]
    print(extracted_table)
    print (extracted_table[np.triu_indices(np.shape(extracted_table)[0], 1)])
    #print(get_submatrix_by_column(table,np.array(indices),indices))


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
from RouToolPa.Tools.Abstract import JavaTool



class CreateSequenceDictionary(JavaTool):

    def __init__(self, java_path="", max_threads=4, jar_path="", max_memory="1g"):
        #JavaTool.__init__(self, "CreateSequenceDictionary.jar", java_path=java_path, max_threads=max_threads,
        #                  jar_path=jar_path, max_memory=max_memory)
        JavaTool.__init__(self, "picard.jar CreateSequenceDictionary", java_path=java_path, max_threads=max_threads,
                          jar_path=jar_path, max_memory=max_memory)

    def make_fasta_dict(self, fasta_file, dict_name):

        options = " R= %s" % fasta_file
        options += " O= %s" % dict_name

        self.execute(options)

    def check_for_fasta_dict(self, fasta_file):

        fasta_path_list = self.split_filename(fasta_file)
        dict_file = "%s%s%s.dict" % (fasta_path_list[0], "/" if fasta_path_list[0] != "" else "", fasta_path_list[1])

        if not os.path.exists(dict_file):
            self.make_fasta_dict(fasta_file, dict_file)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
import os
import shutil
from collections import OrderedDict
from RouToolPa.Tools.Abstract import Tool
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from Bio.Seq import Seq
from RouToolPa.Collections.General import SynDict



class LongRanger(Tool):

    def __init__(self, path="", max_threads=4, max_memory=None):
        Tool.__init__(self, "longranger", path=path, max_threads=max_threads, max_memory=max_memory)

    def parse_wgs_options(self, reference, run_id, fastq_dir=None, sample_prefix=None, description=None,
                          library_name=None, lane_list=None, indice_list=None, project_name=None, variant_calling_mode=None,
                          gatk_jar_path=None, use_somatic_sv_caller=None, precalled_vcf=None, sample_sex=None,
                          variant_calling_only=None, threads=None, max_memory=None):

        options = " wgs"
        options += " --id=%s" % run_id
        if fastq_dir:
            options += " --fastqs=%s" % fastq_dir
        elif sample_prefix:
            options += " --sample=%s" % sample_prefix
        else:
            raise ValueError("Neither directory with fastq files nor sample prefix was set")

        options += " --reference=%s" % reference
        options += " --description=%s" % description if description else ""
        options += " --library=%s" % library_name if library_name else ""

        options += " --lanes=%s" % (lane_list if isinstance(lane_list, str) else ",".join(lane_list)) if lane_list else ""
        options += " --indices=%s" % (indice_list if isinstance(indice_list, str) else ",".join(indice_list)) if indice_list else ""
        options += " --project=%s" % project_name if project_name else ""

        if variant_calling_mode == "gatk":
            options += " --vcmode=gatk:%s" % gatk_jar_path
        else:
            options += " --vcmode=%s" % variant_calling_mode

        options += " --somatic" if use_somatic_sv_caller else ""
        options += " --precalled=%s" % precalled_vcf if precalled_vcf else ""
        options += " --sex=%s" % sample_sex if sample_sex else ""

        options += " --vconly" if variant_calling_only else ""
        options += " --localcores=%i" % (threads if threads else self.threads)
        options += " --localmem=%s" % (str(max_memory) if max_memory else str(self.max_memory)) if max_memory or self.max_memory else ""

        return options

    def run_wgs_analysis(self, reference, run_id, fastq_dir=None, sample_prefix=None, description=None,
                         library_name=None, lane_list=None, indice_list=None, project_name=None, variant_calling_mode=None,
                         gatk_jar_path=None, use_somatic_sv_caller=None, precalled_vcf=None, sample_sex=None,
                         variant_calling_only=None, threads=None, max_memory=None):

        options = self.parse_wgs_options(reference, run_id, fastq_dir=fastq_dir, sample_prefix=sample_prefix,
                                         description=description, library_name=library_name, lane_list=lane_list,
                                         indice_list=indice_list, project_name=project_name,
                                         variant_calling_mode=variant_calling_mode,
                                         gatk_jar_path=gatk_jar_path, use_somatic_sv_caller=use_somatic_sv_caller,
                                         precalled_vcf=precalled_vcf, sample_sex=sample_sex,
                                         variant_calling_only=variant_calling_only, threads=threads,
                                         max_memory=max_memory)

        self.execute(options=options)

    def run_wgs_analysis_from_fastq(self, reference, run_id, fastq_dir, description=None, library_name=None,
                                    lane_list=None, indice_list=None, project_name=None, variant_calling_mode=None,
                                    gatk_jar_path=None, use_somatic_sv_caller=None, precalled_vcf=None, sample_sex=None,
                                    variant_calling_only=None, threads=None, max_memory=None):

        options = self.parse_wgs_options(reference, run_id, fastq_dir=fastq_dir, sample_prefix=None,
                                         description=description, library_name=library_name, lane_list=lane_list,
                                         indice_list=indice_list, project_name=project_name,
                                         variant_calling_mode=variant_calling_mode,
                                         gatk_jar_path=gatk_jar_path, use_somatic_sv_caller=use_somatic_sv_caller,
                                         precalled_vcf=precalled_vcf, sample_sex=sample_sex,
                                         variant_calling_only=variant_calling_only, threads=threads,
                                         max_memory=max_memory)

        self.execute(options=options)

    def prepare_record_dict_reference(self, record_dict, coord_file=None, max_scaffold_length=527000000,
                                      max_scaffold_number=500, polyN_len=500, symbols_to_replace_list=[":"]):
        length_dict = self.get_lengths(record_dict)
        id_list = length_dict.keys()
        number_of_scaffolds = len(length_dict)
        polyN_insersion = "N" * polyN_len
        renamed_scaffolds_syn_dict = SynDict()

        #print id_list
        output_dict = OrderedDict()
        if number_of_scaffolds <= max_scaffold_number:
            return record_dict

        total_length_of_short_scaffolds_with_insertion = 0
        for i in range(499, number_of_scaffolds):
            #print length_dict[id_list[i]]
            total_length_of_short_scaffolds_with_insertion += polyN_len + length_dict[id_list[i]]

        if total_length_of_short_scaffolds_with_insertion < max_scaffold_length:
            merged_record = SeqRecord(id="merged_small_scaffolds", description="merged_records:%i-%i" % (500, number_of_scaffolds),
                                      seq=Seq(polyN_insersion.join(map(lambda record_index: str(record_dict[id_list[record_index]].seq),
                                                                   range(499, number_of_scaffolds)))))
            if coord_file:
                with open(coord_file, "w") as coord_fd:
                    coord_fd.write("#id\tstart\tstop\n")
                    coord_fd.write("%s\t1\t%i\n" % (id_list[499], length_dict[id_list[499]]))
                    prev_end = length_dict[id_list[499]]
                    if number_of_scaffolds > 500:
                        for i in range(500, number_of_scaffolds):
                            coord_fd.write("%s\t%i\t%i\n" % (id_list[i],
                                                              prev_end + 1 + polyN_len,
                                                              prev_end + length_dict[id_list[i]] + polyN_len))
                            prev_end += length_dict[id_list[i]] + polyN_len

            #print merged_record
            for i in range(0, 499):
                if ":" not in id_list[i]:
                    output_dict[id_list[i]] = record_dict[id_list[i]]
                else:
                    new_id = id_list[i]
                    for symbol in symbols_to_replace_list:
                        new_id = new_id.replace(symbol, "_")
                    renamed_scaffolds_syn_dict[id_list[i]] = new_id
                    output_dict[new_id] = SeqRecord(seq=record_dict[id_list[i]].seq, id=new_id, description=record_dict[id_list[i]].description)
            output_dict[merged_record.id] = merged_record
            #print output_dict
            return output_dict, renamed_scaffolds_syn_dict

        # TODO: following code have not been completed yet!!!!!!!!!
        raise ValueError("Following code have not been completed yet!!!!!!!!!")

        number_of_merged_records = 0
        total_number_of_records = number_of_scaffolds
        records_to_merge_list = []

        while total_number_of_records > max_scaffold_number:
            tmp_merge_list = []
            tmp_length = 0
            for i in range(number_of_scaffolds, -1, -1):
                if tmp_length + len(length_dict[id_list[i]]) < max_scaffold_length:
                    pass

    def prepare_reference(self, reference, prepared_reference, renamed_scaffolds_syn_file,
                          max_scaffold_length=527000000, max_scaffold_number=500,
                          polyN_len=500, coord_file=None, symbols_to_replace_list=[":"]):
        """
        Diploid genome - phasing algorithm currently assumes 2 haplotypes
        500 contigs or fewer - if your assembly has more than 500 contigs, concatenate smaller contigs together
        with 500 Ns separating each original contig, until there are fewer than 500 contigs total
        All contigs must be no more than 2^29-1 bp, or 528Mb, in length; this is a limitation of BAM index file format
        All contigs must have no colons or spaces in their names.
        """

        record_dict = self.parse_seq_file(reference, mode="parse")

        prepared_record_dict, renamed_scaffolds_syn_dict = self.prepare_record_dict_reference(record_dict,
                                                                                              max_scaffold_length=max_scaffold_length,
                                                                                              max_scaffold_number=max_scaffold_number,
                                                                                              polyN_len=polyN_len,
                                                                                              coord_file=coord_file,
                                                                                              symbols_to_replace_list=symbols_to_replace_list)
        SeqIO.write(self.record_from_dict_generator(prepared_record_dict), prepared_reference, format='fasta')
        renamed_scaffolds_syn_dict.write(renamed_scaffolds_syn_file)
        self.index_reference(prepared_reference)

    def index_reference(self, reference):

        options = " mkref %s" % reference

        self.execute(options=options)



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Collections.General import IdSet
from RouToolPa.Routines import FileRoutines


class RepeatMasker(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "RepeatMasker", path=path, max_threads=max_threads)
        self.repeat_classes_used_for_gene_annotation = ["DNA",
                                                        "DNA\?",
                                                        "LINE",
                                                        "LINE\?",
                                                        "LTR",
                                                        "LTR\?",
                                                        "RC",
                                                        "RC\?",
                                                        "Retroposon",
                                                        "Retroposon\?",
                                                        "SINE",
                                                        "SINE\?",
                                                        "Helitron",
                                                        "Helitron\?"
                                                        ]
        self.repeat_classes_used_for_gene_annotation_expanded_set = self.repeat_classes_used_for_gene_annotation + ["Unknown",
                                                                                                                    "Other",
                                                                                                                    "ARTEFACT"]

    @staticmethod
    def convert_rm_out_to_gff(input_file, output_file, annotated_repeat_classes_file, annotated_repeat_families_file):
        repeat_classes_set = IdSet()
        repeat_families_set = IdSet()
        with open(input_file, "r") as in_fd:
            for i in range(0, 3):
                in_fd.readline()

            with open(output_file, "w") as out_fd:
                for line in in_fd:
                    tmp = line.strip().split()
                    strand = "+" if tmp[8] == "+" else "-"
                    repeat_class_family = tmp[10].split("/")
                    if len(repeat_class_family) == 1:
                        repeat_class_family.append(".")
                    repeat_classes_set.add(repeat_class_family[0])
                    repeat_families_set.add("/".join(repeat_class_family))
                    parameters = "Class=%s;Family=%s;Matching_repeat=%s;SW_score=%s;Perc_div=%s;Perc_del=%s;Pers_ins=%s" \
                                 % (repeat_class_family[0], repeat_class_family[1],
                                    tmp[9], tmp[0], tmp[1], tmp[2], tmp[3])
                    out_fd.write("%s\tRepeatMasker\trepeat\t%s\t%s\t.\t%s\t.\t%s\n" % (tmp[4], tmp[5], tmp[6], strand, parameters))
        repeat_classes_set.write(annotated_repeat_classes_file)
        repeat_families_set.write(annotated_repeat_families_file)

    @staticmethod
    def extract_annotated_repeat_types_from_gff(gff_file, annotated_repeat_classes_file):
        sed_string = "sed -r 's/.*Class=(.*);Family.*/\1/' %s | sort | uniq > %s" % (gff_file,
                                                                                     annotated_repeat_classes_file)
        # awk variant of string
        # awk -F'\t' '{print $9}' repeatmasker.selected_repeat_classes.gff | awk -F';' '{print $1}' | awk -F'=' '{print $2}' | sort | uniq
        os.system(sed_string)

    def extract_repeats_used_for_gene_annotation(self, input_gff, output_gff,
                                                 use_expanded_set=False,
                                                 retain_unknown_repeats=False):
        if use_expanded_set:
            grep_pattern = "|".join(self.repeat_classes_used_for_gene_annotation_expanded_set)
        elif retain_unknown_repeats:
            grep_pattern = "|".join(self.repeat_classes_used_for_gene_annotation + ["Unknown", ])
        else:
            grep_pattern = "|".join(self.repeat_classes_used_for_gene_annotation)

        grep_string = "grep -P '%s'" % grep_pattern
        grep_string += " %s" % input_gff
        grep_string += " > %s" % output_gff
        self.execute(cmd=grep_string)

    def extract_repeats_from_database(self, output_file, species=None, clade=None, stat_mode=None):

        options = " -species %s" % species if species else ""
        options += " -clade %s" % clade if clade else ""
        options += " -stat" if stat_mode else ""
        #options += " | awk 'NR > 1 {print $0}'" if not stat_mode else ""
        options += " > %s" % output_file

        self.execute(options=options, cmd="queryRepeatDatabase.pl")

    def mask(self, list_of_fasta_files, output_dir="./", soft_masking=True, engine=None,
             search_speed=None, no_low_complexity=None, only_low_complexity=None,
             no_interspersed=None, only_interspersed=None, no_rna=None, only_alu=None, custom_library=None,
             species=None, html_output=False, ace_output=False, gff_output=False):

        if species and custom_library:
            tmp_repeat_file = "%s/%s.repeats.tmp.fa" % (output_dir, species)
            tmp_repeats_all_file = "%s/all.repeats.tmp.fasta" % output_dir
            self.extract_repeats_from_database(tmp_repeat_file, species=species)

            cmd = "cat %s %s > %s" % (tmp_repeat_file, custom_library, tmp_repeats_all_file)
            self.execute(cmd=cmd)

        options = " -pa %i" % self.threads
        options += " -e %s" % engine if engine else ""

        if search_speed == "slow":
            options += " -s"
        elif search_speed == "quick":
            options += " -q"
        elif search_speed == "rush":
            options += " -qq"

        options += " -nolow" if no_low_complexity else ""
        options += " -low" if only_low_complexity else ""
        options += " -noint" if no_interspersed else ""
        options += " -int" if only_interspersed else ""
        options += " -norna" if no_rna else ""
        options += " -alu" if only_alu else ""

        if species and custom_library:
            options += " -lib %s" % tmp_repeats_all_file
        elif custom_library:
            options += " -lib %s" % custom_library if custom_library else ""
        elif species:
            options += " -species %s" % species if species else ""

        options += " -dir %s" % output_dir
        options += " -html" if html_output else ""
        options += " -ace" if ace_output else ""
        options += " -gff" if gff_output else ""
        options += " -xsmall" if soft_masking else ""

        options += " " + (list_of_fasta_files if isinstance(list_of_fasta_files, str) else  " ".join(FileRoutines.make_list_of_path_to_files(list_of_fasta_files)))

        self.execute(options=options)

        """
    -source
        Includes for each annotation the HSP "evidence". Currently this
        option is only available with the "-html" output format listed
        below.

    -u  Creates an additional annotation file not processed by
        ProcessRepeats

    -xm Creates an additional output file in cross_match format (for
        parsing)



    -div [number]
        Masks only those repeats < x percent diverged from consensus seq


    -cutoff [number]
        Sets cutoff score for masking repeats when using -lib (default 225)



    Contamination options

    -is_only
        Only clips E coli insertion elements out of fasta and .qual files

    -is_clip
        Clips IS elements before analysis (default: IS only reported)

    -no_is
        Skips bacterial insertion element check

    Running options

    -gc [number]
        Use matrices calculated for 'number' percentage background GC level

    -gccalc
        RepeatMasker calculates the GC content even for batch files/small
        seqs

    -frag [number]
        Maximum sequence length masked without fragmenting (default 60000,
        300000 for DeCypher)

    -nocut
        Skips the steps in which repeats are excised

    -noisy
        Prints search engine progress report to screen (defaults to .stderr
        file)

    -nopost
        Do not postprocess the results of the run ( i.e. call ProcessRepeats
        ). NOTE: This options should only be used when ProcessRepeats will
        be run manually on the results.

    output options



    -a(lignments)
        Writes alignments in .align output file

    -inv
        Alignments are presented in the orientation of the repeat (with
        option -a)

    -lcambig
        Outputs ambiguous DNA transposon fragments using a lower case name.
        All other repeats are listed in upper case. Ambiguous fragments
        match multiple repeat elements and can only be called based on
        flanking repeat information.

    -small
        Returns complete .masked sequence in lower case


    -poly
        Reports simple repeats that may be polymorphic (in file.poly)


    -no_id
        Leaves out final column with unique ID for each element (was
        default)

    -e(xcln)
        Calculates repeat densities (in .tbl) excluding runs of >=20 N/Xs in
        the query
        """